// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.Comparator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.swt.IFocusService;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.SearchManager;

/**
 * <p>BulkEditorSearchComposite class.</p>
 *
 * @author p.ciardelli
 * @author e.-m.lee
 * @author n.hoffmann
 * @created 17.08.2009
 * @version 1.0
 */
public class BulkEditorSearch {
	
	/**
	 * 
	 */
	private static final String SEARCH = "Search";
	
	private static final String DEFAULT_TEXT = String.format("Use \'%s\' for wildcard searching", SearchManager.WILDCARD);
	
	private final BulkEditor editor;
	
	private Text text;
	private BulkEditorSortCombo sortCombo;
	
	private Button button;
	
	
	public Object ORDER_BY = new Object();
	
	/**
	 * <p>Constructor for BulkEditorSearchComposite.</p>
	 *
	 * @param parent a {@link org.eclipse.swt.widgets.Composite} object.
	 * @param style a int.
	 * @param editor a {@link org.eclipse.ui.IEditorPart} object.
	 */
	public BulkEditorSearch(BulkEditor editor, Composite parent, int style) {
		this.editor = editor;
		
		createControl(parent, style);
	}

	/*
	 * Creates the search control.
	 */
	/**
	 * <p>createControl</p>
	 */
	protected void createControl(Composite parent, int style) {
	
		final Composite container = new Composite(parent, style);
		GridData gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
		container.setLayoutData(gridData);
		container.setLayout(new GridLayout(5, false));
		
		createSearchTextField(container, SWT.BORDER | SWT.SINGLE | SWT.FULL_SELECTION);
		
		createSortCombo(container, style);
		
		button = new Button(container, SWT.PUSH);
		button.setText(SEARCH);
		button.addSelectionListener(new SelectionAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				updateEditorInput();
			}
		});
		
		registerAtFocusService();
	}

	/**
	 * @param container
	 * @param style
	 */
	private void createSortCombo(Composite parent, int style) {
		sortCombo = new BulkEditorSortCombo(parent, editor.getEditorInput().getSortProviders());
		
	}

	/**
	 * Handles focus changes for the textfield.
	 */
	private void registerAtFocusService() {
		IFocusService focusService = 
			(IFocusService) PlatformUI.getWorkbench().getService(IFocusService.class);
		if (focusService != null) {
			focusService.addFocusTracker(text, "bulkeditor.textControlId");
		}
	}


	/**
	 * Creates the search textfield.
	 */
	private void createSearchTextField(Composite parent, int style) {
		final Label label = new Label(parent, SWT.NONE);
		label.setText("Title Cache");
		
		text = new Text(parent, style);
		text.setText(DEFAULT_TEXT);
		text.setForeground(BulkEditorUtil.getColor(Resources.SEARCH_VIEW_FOREGROUND));
		
		text.addFocusListener(new FocusListener() {

			public void focusGained(FocusEvent e) {
				text.setForeground(BulkEditorUtil.getColor(Resources.SEARCH_VIEW_FOCUS));
				if (DEFAULT_TEXT.equals(text.getText())) {
					text.setText("");
				}
			}

			public void focusLost(FocusEvent e) {
				if (CdmUtils.isEmpty(text.getText())) {
					text.setForeground(BulkEditorUtil.getColor(Resources.SEARCH_VIEW_FOREGROUND));
					text.setText(DEFAULT_TEXT);		
				} 
			}
		});
		
		text.addKeyListener(new KeyAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.KeyAdapter#keyReleased(org.eclipse.swt.events.KeyEvent)
			 */
			@Override
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.CR) {
					updateEditorInput();
				}
			}
		});
		
		GridData gridData = new GridData(SWT.FILL, SWT.CENTER, true, false);
		text.setLayoutData(gridData);
	}


	/*
	 * Shows the results of the search.
	 */
	private void updateEditorInput() {
		
		String searchString = getSearchString().trim();
				
		if(DEFAULT_TEXT.equals(searchString) || CdmUtils.isEmpty(searchString)){
			return;
		}			
		
		BulkEditorQuery query = new BulkEditorQuery(getSearchString(), getComparator());
		editor.performSearch(query);
	}
	
	/*
	 * Returns the current string in the search textfield.
	 * @return the content of the textfield
	 */
	/**
	 * <p>getSearchString</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public String getSearchString() {
		return text.getText().trim();
	}
	
	/*
	 * 
	 */
	/**
	 * <p>getComparator</p>
	 *
	 * @return a {@link java.util.Comparator} object.
	 */
	public Comparator getComparator() {
		return sortCombo.getSelection();
	};
	
	/*
	 * 
	 */

	/**
	 * 
	 */
	public void setFocus() {
		if(text != null && ! text.isDisposed()){
			text.setFocus();
		}
	}
}
